#!/usr/bin/env python

import sys, string
from qt import *


fileopen = [
'    16    13        5            1',
'. c #040404',
'# c #808304',
'a c #bfc2bf',
'b c #f3f704',
'c c #f3f7f3',
'aaaaaaaaa...aaaa',
'aaaaaaaa.aaa.a.a',
'aaaaaaaaaaaaa..a',
'a...aaaaaaaa...a',
'.bcb.......aaaaa',
'.cbcbcbcbc.aaaaa',
'.bcbcbcbcb.aaaaa',
'.cbcb...........',
'.bcb.#########.a',
'.cb.#########.aa',
'.b.#########.aaa',
'..#########.aaaa',
'...........aaaaa'
]

filesave = [
'    14    14        3            1',
'. c #040404',
'# c #808304',
'a c #bfc2bf',
'..............',
'.#.aaaaaaaa.a.',
'.#.aaaaaaaa...',
'.#.aaaaaaaa.#.',
'.#.aaaaaaaa.#.',
'.#.aaaaaaaa.#.',
'.#.aaaaaaaa.#.',
'.##........##.',
'.############.',
'.##.........#.',
'.##......aa.#.',
'.##......aa.#.',
'.##......aa.#.',
'a.............'
]

fileprint = [
'    16    14        5            1',
'. c #000000',
'# c #848284',
'a c #c6c3c6',
'b c #ffff00',
'c c #ffffff',
'aaaaa.........aa',
'aaaa.cccccccc.aa',
'aaaa.c.....c.aaa',
'aaa.cccccccc.aaa',
'aaa.c.....c....a',
'aa.cccccccc.a.a.',
'a..........a.a..',
'.aaaaaaaaaa.a.a.',
'.............aa.',
'.aaaaaa###aa.a.a',
'.aaaaaabbbaa...a',
'.............a.a',
'a.aaaaaaaaa.a.aa',
'aa...........aaa'
]


fileOpenText = """Click this button to open a new file.

You can also select the Open command from the File menu."""

fileSaveText = """Click this button to save the file you are editing.
You will be prompted for a filename.

You can also select the Save command from the File menu.

Note that implementing this function is left as an exercise for the reader."""

filePrintText = """Click this button to print the file you are editing.

You can also select the Print command from the File menu."""

class ApplicationWindow(QMainWindow):
  def __init__(self, *args):
    apply(QMainWindow.__init__,(self,)+args)

    self.printer = QPrinter()
    self.printer.setMinMax(1,10)

    self.fileTools = QToolBar(self, 'file operations')

    openIcon = QPixmap(fileopen)
    self.fileOpen = QToolButton(openIcon, 'Open File', '',
			  self.loadDoc,
			  self.fileTools, 'open file')

    saveIcon = QPixmap(filesave)
    self.fileSave = QToolButton(saveIcon, 'Save File', '',
			  self.saveDoc,
			  self.fileTools, 'save file')
	      
    printIcon = QPixmap(fileprint)
    self.filePrint = QToolButton(printIcon, 'Print File', '',
    			  self.printDoc,
			  self.fileTools, 'print file')

    QWhatsThis.whatsThisButton(self.fileTools)
    QWhatsThis.add(self.fileOpen, fileOpenText, 0)
    QWhatsThis.add(self.fileSave, fileSaveText, 0)
    QWhatsThis.add(self.filePrint, filePrintText, 0)

    self.file = QPopupMenu()
    self.menuBar().insertItem('&File', self.file)

    self.file.insertItem('New', self.newDoc, CTRL+Key_N)
    self.file.insertItem('Open', self.loadDoc, CTRL+Key_O)
    self.file.insertItem('Save', self.saveDoc, CTRL+Key_S)
    self.file.insertSeparator()
    self.file.insertItem('Print', self.printDoc, CTRL+Key_P)
    self.file.insertSeparator()
    self.file.insertItem('Close', self.closeDoc, CTRL+Key_W)
    self.file.insertItem('Quit', qApp.quit, CTRL+Key_Q)

    self.controls = controls = QPopupMenu()
    self.menuBar().insertItem('&Controls', controls)

    self.mb = controls.insertItem('Menu Bar', self.toggleMenuBar, CTRL+Key_M)
    a = QAccel(self)
    a.connectItem(a.insertItem(CTRL+Key_M), self.toggleMenuBar)

    self.tb = controls.insertItem('Tool Bar', self.toggleToolBar, CTRL+Key_T)
    self.sb = controls.insertItem('Status Bar', self.toggleStatusBar, CTRL+Key_B)
    controls.setCheckable(1)
    controls.setItemChecked(self.mb, 1)
    controls.setItemChecked(self.tb, 1)
    controls.setItemChecked(self.sb, 1)
 
    self.e = e = QMultiLineEdit(self, 'editor')
    e.setFocus()
    self.setCentralWidget(e)
    self.statusBar().message('Read', 2000)

  def newDoc(self):
    ed = ApplicationWindow()
    ed.resize(400, 400)
    ed.show()

  def loadDoc(self, fileName=None):
    if not fileName:
      fn = QFileDialog.getOpenFileName('', '', self)
      if not fn.isEmpty():
	self.loadDoc(`fn`)
      else:
        self.statusBar().message('Loading aborted', 2000)
    else:
      f = open(fileName, 'r')
      for l in f.readlines():
        self.e.append(string.strip(l))
      f.close()

      self.e.setAutoUpdate(1)
      self.e.repaint()
      self.setCaption(fileName)

      self.statusBar().message('Loaded document %s' % fileName, 2000)

  def saveDoc(self):
    self.statusBar().message('File->Save is not implemented')
    QMessageBox.information(None, 'Note', 'Left as an exercise for the user.', QMessageBox.Ok)

  def printDoc(self):
    MARGIN = 10
    pageNo = 1
    if self.printer.setup(self):
      self.statusBar().message('Printing...')
      p = QPainter()
      p.begin(self.printer)
      p.setFont(e.font())
      yPos = 0
      fm = p.fontMetrics()
      metrics = QPaintDeviceMetrics(self.printer)
      for i in range(e.numLines):
        if MARGIN + yPos > metrics.height() - MARGIN:
	  pageNo = pageNo + 1
	  self.statusBar().message('Printing (page %d)...' % pageNo)
	  self.printer.newPage()
	  yPos = 0
        p.drawText(MARGIN, MARGIN+yPos,
	           metrics.width(), fm.lineSpacing(),
		   ExpandTabs|DontClip,
		   e.textLine(i))
	yPos = yPos + fm.lineSpacing()
      p.end()
      self.statusBar().message('Printing completed', 2000)
    else:
      self.statusBar().message('Printing aborted', 2000)
 
 
  def closeDoc(self):
    self.close(1)  # close and DELETE!

  def toggleMenuBar(self):
    if self.menuBar().isVisible():
      self.menuBar().hide()
      self.controls.setItemChecked(self.mb, 0)
    else:
      self.menuBar().show()
      self.controls.setItemChecked(self.mb, 1)


  def toggleStatusBar(self):
    if self.statusBar().isVisible():
      self.statusBar().hide()
      self.controls.setItemChecked(self.sb, 0)
    else:
      self.statusBar().show()
      self.controls.setItemChecked(self.sb, 1)

  def toggleToolBar(self):
    if self.fileTools.isVisible():
      self.fileTools.hide()
      self.controls.setItemChecked(self.tb, 0)
    else:
      self.fileTools.show()
      self.controls.setItemChecked(self.tb, 1)

a = QApplication(sys.argv)
mw = ApplicationWindow()
a.setMainWidget(mw)
mw.setCaption('Document 1')
mw.show()
a.connect(a, SIGNAL('lastWindowClosed()'), a, SLOT('quit()'))
a.exec_loop()
